@@ -1,8 +1,7 @@
-use std::io::Read;
-
 use bytes::{Buf, BufMut, Bytes, BytesMut};
 use flate2::read::GzDecoder;
 use hyper::body::HttpBody;
+use io::Read;
 use rand::{thread_rng, Rng};
 
 #[allow(unused_imports)]
@@ -11,7 +10,6 @@ use crate::{
 	hyper_utils,
 	resolver::{Adaptor as ResolverAdaptor, Resolver},
 	types::*,
-	ParserProcessor,
 };
 
 pub(crate) type HttpClient<R> = hyper::Client<HttpConnector<R>>;
@@ -31,7 +29,7 @@ pub(crate) struct TaskProcessor<JS: JobStateValues, TS: TaskStateValues, R: Reso
 
 	tx:             Sender<JobUpdate<JS, TS>>,
 	tasks_rx:       Receiver<Arc<Task>>,
-	pp:             ParserProcessor,
+	parse_tx:       Sender<ParserTask>,
 	client_factory: Box<dyn ClientFactory<R> + Send + Sync + 'static>,
 	resolver:       Arc<R>,
 }
@@ -41,7 +39,7 @@ impl<JS: JobStateValues, TS: TaskStateValues, R: Resolver> TaskProcessor<JS, TS,
 		job: ResolvedJob<JS, TS>,
 		tx: Sender<JobUpdate<JS, TS>>,
 		tasks_rx: Receiver<Arc<Task>>,
-		pp: ParserProcessor,
+		parse_tx: Sender<ParserTask>,
 		client_factory: Box<dyn ClientFactory<R> + Send + Sync + 'static>,
 		resolver: Arc<R>,
 		term_on_error: bool,
@@ -54,7 +52,7 @@ impl<JS: JobStateValues, TS: TaskStateValues, R: Resolver> TaskProcessor<JS, TS,
 			job,
 			tx,
 			tasks_rx,
-			pp,
+			parse_tx,
 			client_factory,
 			resolver,
 		}
@@ -118,7 +116,7 @@ impl<JS: JobStateValues, TS: TaskStateValues, R: Resolver> TaskProcessor<JS, TS,
 		let req_fut = client.request(req.body(hyper::Body::default()).context("could not construct http request")?);
 		let resp = timeout(*self.job.settings.load_timeout, req_fut).await.map_err(|_| Error::LoadTimeout)?;
 
-		let resp = resp.with_context(|| "cannot make http get")?;
+		let resp = resp.context("cannot make http get")?;
 		status_metrics.duration = t.elapsed();
 		let rs = resp.status();
 
@@ -189,6 +187,8 @@ impl<JS: JobStateValues, TS: TaskStateValues, R: Resolver> TaskProcessor<JS, TS,
 		status: HttpStatus,
 		reader: Box<dyn io::Read + Sync + Send>,
 	) -> Result<FollowData> {
+		let (parse_res_tx, parse_res_rx) = bounded_ch::<ParserResponse>(1);
+
 		let task_state = Arc::new(Mutex::new(Some(TS::default())));
 		let links = Arc::new(Mutex::new(Some(Vec::<Link>::new())));
 
@@ -202,6 +202,7 @@ impl<JS: JobStateValues, TS: TaskStateValues, R: Resolver> TaskProcessor<JS, TS,
 
 			move || -> Result<FollowData> {
 				let t = Instant::now();
+
 				let document = select::document::Document::from_read(reader).context("cannot read html document")?;
 
 				for expander in task_expanders.iter() {
@@ -231,9 +232,13 @@ impl<JS: JobStateValues, TS: TaskStateValues, R: Resolver> TaskProcessor<JS, TS,
 			}
 		};
 
-		let parser_response = self.pp.spawn(ParserTask { payload: Box::new(payload), time: Instant::now() }).await;
+		let _ = self
+			.parse_tx
+			.send_async(ParserTask { payload: Box::new(payload), time: Instant::now(), res_tx: parse_res_tx })
+			.await;
 
-		let follow = parser_response.payload?;
+		let parser_response = parse_res_rx.recv_async().await.context("cannot follow html document")?;
+		let follow_data = parser_response.payload?;
 		{
 			let mut task_state = task_state.lock().unwrap();
 			self.job.ctx.task_state = task_state.take().unwrap();
@@ -242,7 +247,7 @@ impl<JS: JobStateValues, TS: TaskStateValues, R: Resolver> TaskProcessor<JS, TS,
 			let mut links = links.lock().unwrap();
 			self.job.ctx.push_links(links.take().unwrap());
 		}
-		Ok(follow)
+		Ok(follow_data)
 	}
 
 	async fn process_task(
@@ -253,14 +258,7 @@ impl<JS: JobStateValues, TS: TaskStateValues, R: Resolver> TaskProcessor<JS, TS,
 	) -> Result<JobProcessing> {
 		let resolve_data = self.resolve(Arc::clone(&task)).await?;
 
-		let mut status = JobProcessing {
-			resolve_data,
-			head_status: StatusResult::None,
-			status: StatusResult::None,
-			load: LoadResult::None,
-			follow: FollowResult::None,
-			links: vec![],
-		};
+		let mut status = JobProcessing::new(resolve_data);
 
 		if task.link.target == LinkTarget::JustResolveDNS {
 			return Ok(status)
@@ -319,22 +317,21 @@ impl<JS: JobStateValues, TS: TaskStateValues, R: Resolver> TaskProcessor<JS, TS,
 		Ok(status)
 	}
 
-	pub(crate) fn go(&mut self, n: usize) -> PinnedTask {
+	pub(crate) fn go<'a>(mut self, n: usize) -> PinnedTask<'a> {
 		TracingTask::new(span!(n=n, url=%self.job.url), async move {
 			let (client, mut stats) = self.client_factory.make();
 
+			let mut timeout = self.job.ctx.timeout_remaining(*self.job.settings.job_hard_timeout);
+
 			while let Ok(task) = self.tasks_rx.recv_async().await {
 				if self.tasks_rx.is_disconnected() {
 					break
 				}
 
-				let timeout = self.job.ctx.timeout_remaining(*self.job.settings.job_hard_timeout);
-
 				stats.reset();
 
 				tokio::select! {
 					mut status_r = self.process_task(Arc::clone(&task), &client, &stats) => {
-						let ctx = self.job.ctx.clone();
 						if let Ok(ref mut status_data) = status_r {
 							status_data.links.extend(self.job.ctx.consume_links());
 						}
@@ -342,10 +339,10 @@ impl<JS: JobStateValues, TS: TaskStateValues, R: Resolver> TaskProcessor<JS, TS,
 						let _ = self.tx.send_async(JobUpdate {
 							task,
 							status: JobStatus::Processing(status_r),
-							ctx,
+							ctx: self.job.ctx.clone(),
 						}).await;
 					}
-					_ = timeout => break
+					_ = &mut timeout => break
 				}
 
 				if self.job.settings.delay.as_millis() > 0 || self.job.settings.delay_jitter.as_millis() > 0 {
@@ -354,10 +351,9 @@ impl<JS: JobStateValues, TS: TaskStateValues, R: Resolver> TaskProcessor<JS, TS,
 						Duration::from_millis(rng.gen_range(0..self.job.settings.delay_jitter.as_millis()) as u64)
 					};
 
-					let timeout = self.job.ctx.timeout_remaining(*self.job.settings.job_hard_timeout);
 					tokio::select! {
-						_ = time::sleep(*self.job.settings.delay+jitter_ms) => {}
-						_ = timeout => break
+						_ = time::sleep(*self.job.settings.delay + jitter_ms) => {}
+						_ = &mut timeout => break
 					}
 				}
 			}
